#! /bin/bash
#AUBO ROBOT 程序启动脚本
#author:cdj

#***********************************************************************
#函数功能: 获取当前脚本所在的绝对路径  
#参数说明: 1个参数   arg1:执行该程序时的命令行第一个参数
#
#说   明: 
#***********************************************************************
get_this_exec_file_path(){
	
	dirname $0|grep "^/" >/dev/null
	
	if [ $? -eq 0 ];then
		this_dir=`dirname $0`
	else
		dirname $0|grep "^\." >/dev/null
		retval=$?
		if [ $retval -eq 0 ];then
			this_dir=`dirname $0|sed "s#^.#$this_dir#"`
		else
			this_dir=`dirname $0|sed "s#^#$this_dir/#"`
		fi
	fi
	
	this_dir=${this_dir}"/"
	
	if [ $? = "1" ] ;then  
        return 1  
    else  
        return 0  
    fi
}


####杀死某个应用程序
kill_process() {
	
	processName=$1
	
	result=$(ps -ef | grep -v grep | grep $processName)

	if [ -n "$result" ]; then
		killall "$processName"
	fi
	
	#pid=`ps | grep "zs" | grep -v grep | head -n 1 | awk '{print $1}'`
	#kill -9 $pid
}
######******************************************************************




run_or_kill_robotServer(){
	
	type=$1
	RobotServerProgramName="auboControllerServer"
	RobotServerProgramPath="/root/AuboRobotWorkSpace/RobotServer/bin/"
	
	if [ "$type" == "run" ];then
		kill_process "$RobotServerProgramName"
	
		#启动服务器
		cd "$RobotServerProgramPath"
		"$RobotServerProgramPath"${RobotServerProgramName} &
		cd -
	fi
	
	if [ "$type" == "kill" ];then
		kill_process "$RobotServerProgramName"
	fi

}


run_or_kill_robotSplash(){
	
	type=$1
	
	if [ "$type" == "run" ];then
		kill_process "RobotSplashExec"
		
		#启动1:RobotSplash启动
		cd ${this_dir}"robotsplash"
		splashExec=${this_dir}"robotsplash/RobotSplashExec"
		echo "splashExec:$splashExec"
		"$splashExec" &
		cd -
		echo "-----------"
	fi
	
	if [ "$type" == "kill" ];then
		kill_process "RobotSplashExec"
	fi
}


run_or_kill_robotKeyboard(){
	
	type=$1
	
	#机械臂示教器程序输入法插件的程序名　　　　　
	TeachPendantProgramKeyboardPluginName="Keyboard"
	TeachPendantProgramKeyboardPluginPath="/root/AuboRobotWorkSpace/teachpendant/bin/"
	
	if [ "$type" == "run" ];then
		kill_process "$TeachPendantProgramKeyboardPluginName"
	
		#启动服务器
		cd "$TeachPendantProgramKeyboardPluginPath"
		"$TeachPendantProgramKeyboardPluginPath"${TeachPendantProgramKeyboardPluginName}  &
		cd -
	fi
	
	if [ "$type" == "kill" ];then
		kill_process "$TeachPendantProgramKeyboardPluginName"
	fi
}



run_or_kill_robotAUBOPE(){
	
	type=$1
	
	TeachPendantProgramName="AUBOPE"
	TeachPendantProgramPath="/root/AuboRobotWorkSpace/teachpendant/bin/"
	
	if [ "$type" == "run" ];then
		kill_process "$TeachPendantProgramName"
	
		#启动服务器
		cd "$TeachPendantProgramPath"
		"$TeachPendantProgramPath"${TeachPendantProgramName}  &
		cd -
	fi
	
	if [ "$type" == "kill" ];then
		kill_process "$TeachPendantProgramName"
	fi
}


#初始化 this_dir 变量
this_dir=`pwd`
get_this_exec_file_path $0


#机械臂服务器端口号
RobotServerPort=8899
RobotServerProgramName="auboControllerServer"


#准备工作:杀死之前已经启动的程序
run_or_kill_robotSplash   "kill"
run_or_kill_robotKeyboard "kill"
run_or_kill_robotAUBOPE   "kill"
#run_or_kill_robotServer   "kill"


#运行Splash程序
run_or_kill_robotSplash "run"


####启动服务器************************************************************
serviceStartupFinish="false"

#执行ldconfig
ldconfig

for i in $(seq 1 300)
do
	#判断服务器的端口是否处于监听状态
	if [ ! -z "$(netstat -anl |grep $RobotServerPort |grep LISTEN)" ];then
		#服务器的端口已经处于监听状态
		echo "Robot server running, port listened"
		serviceStartupFinish="true"
		break;
	else
		#端口未监听	
		if [ ! -z "$(ps -ef | grep -v grep |grep $RobotServerProgramName)" ];then
			#服务器程序已启动	　尝试连接服务器
			echo "Robot server running, port not listen"		
			echo -e "\n"|telnet localhost $RobotServerPort |grep Connected
		else
			#服务器程序未启动　　运行服务器
			echo "Robot server not running"
			#后台启动服务器程序
			run_or_kill_robotServer "run"
		fi
		#延时1秒
		sleep 1
	fi
done
##########################################################################


if [ "$serviceStartupFinish" == "true" ];then
	echo "服务器已经成功启动"
	
	#启动软键盘
	run_or_kill_robotKeyboard "run"
	
	sleep 1
	
	#启动示教器程序
	run_or_kill_robotAUBOPE   "run"
	
	sleep 4
	run_or_kill_robotSplash  "kill"
fi

dateStr="auborobotstartup-`date +%y-%m-%d-%H-%M-%S`"
echo "$dateStr" >> "$this_dir/log.txt"


exit 0
